import { Injectable } from '@angular/core';
import { assign, cloneDeep } from 'lodash-es';
import { FuseMockApiService, FuseMockApiUtils } from '@fuse/lib/mock-api';
import { recommendations as recommendationsData } from './data';

@Injectable({
  providedIn: 'root',
})
export class MemberRecommendationMockApi {
  private _recommendations: any[] = recommendationsData;

  /**
   * Constructor
   */
  constructor(private _fuseMockApiService: FuseMockApiService) {
    // Register Mock API handlers
    this.registerHandlers();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Register Mock API handlers
   */
  registerHandlers(): void {
    // -----------------------------------------------------------------------------------------------------
    // @ Recommendations - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/member/recommendation/recommendations', 300)
      .reply(({ request }) => {
        // Get available queries
        const search = request.params.get('search');
        const sort = request.params.get('sort') || 'signinId';
        const order = request.params.get('order') || 'asc';
        const page = parseInt(request.params.get('page') ?? '1', 10);
        const size = parseInt(request.params.get('size') ?? '10', 10);

        // Clone the recommendations
        let recommendations: any[] | null = cloneDeep(this._recommendations);

        // Sort the recommendations
        if (sort === 'signinId' || sort === 'nickname' || sort === 'state') {
          recommendations.sort((a, b) => {
            const fieldA = a[sort].toString().toUpperCase();
            const fieldB = b[sort].toString().toUpperCase();
            return order === 'asc'
              ? fieldA.localeCompare(fieldB)
              : fieldB.localeCompare(fieldA);
          });
        } else {
          recommendations.sort((a, b) =>
            order === 'asc' ? a[sort] - b[sort] : b[sort] - a[sort]
          );
        }

        // If search exists...
        if (search) {
          // Filter the recommendations
          recommendations = recommendations.filter(
            (contact: any) =>
              contact.name &&
              contact.name.toLowerCase().includes(search.toLowerCase())
          );
        }

        // Paginate - Start
        const recommendationsLength = recommendations.length;

        // Calculate pagination details
        const begin = page * size;
        const end = Math.min(size * (page + 1), recommendationsLength);
        const lastPage = Math.max(Math.ceil(recommendationsLength / size), 1);

        // Prepare the pagination object
        let pagination = {};

        // If the requested page number is bigger than
        // the last possible page number, return null for
        // recommendations but also send the last possible page so
        // the app can navigate to there
        if (page > lastPage) {
          recommendations = null;
          pagination = {
            lastPage,
          };
        } else {
          // Paginate the results by size
          recommendations = recommendations.slice(begin, end);

          // Prepare the pagination mock-api
          pagination = {
            length: recommendationsLength,
            size: size,
            page: page,
            lastPage: lastPage,
            startIndex: begin,
            endIndex: end - 1,
          };
        }

        // Return the response
        return [
          200,
          {
            recommendations,
            pagination,
          },
        ];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Recommendation - GET
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onGet('api/apps/member/recommendation/recommendation')
      .reply(({ request }) => {
        // Get the id from the params
        const id = request.params.get('id');

        // Clone the recommendations
        const recommendations = cloneDeep(this._recommendations);

        // Find the recommendation
        const recommendation = recommendations.find(
          (item: any) => item.id === id
        );

        // Return the response
        return [200, recommendation];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Recommendation - POST
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPost('api/apps/member/recommendation/recommendation')
      .reply(() => {
        // Generate a new recommendation
        const newRecommendation = {
          id: FuseMockApiUtils.guid(),
          category: '',
          name: 'A New User',
          description: '',
          tags: [],
          sku: '',
          barcode: '',
          brand: '',
          vendor: '',
          stock: '',
          reserved: '',
          cost: '',
          basePrice: '',
          taxPercent: '',
          price: '',
          weight: '',
          thumbnail: '',
          images: [],
          active: false,
        };

        // Unshift the new recommendation
        this._recommendations.unshift(newRecommendation);

        // Return the response
        return [200, newRecommendation];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Recommendation - PATCH
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onPatch('api/apps/member/recommendation/recommendation')
      .reply(({ request }) => {
        // Get the id and recommendation
        const id = request.body.id;
        const recommendation = cloneDeep(request.body.recommendation);

        // Prepare the updated recommendation
        let updatedRecommendation = null;

        // Find the recommendation and update it
        this._recommendations.forEach((item, index, recommendations) => {
          if (item.id === id) {
            // Update the recommendation
            recommendations[index] = assign(
              {},
              recommendations[index],
              recommendation
            );

            // Store the updated recommendation
            updatedRecommendation = recommendations[index];
          }
        });

        // Return the response
        return [200, updatedRecommendation];
      });

    // -----------------------------------------------------------------------------------------------------
    // @ Recommendation - DELETE
    // -----------------------------------------------------------------------------------------------------
    this._fuseMockApiService
      .onDelete('api/apps/member/recommendation/recommendation')
      .reply(({ request }) => {
        // Get the id
        const id = request.params.get('id');

        // Find the recommendation and delete it
        this._recommendations.forEach((item, index) => {
          if (item.id === id) {
            this._recommendations.splice(index, 1);
          }
        });

        // Return the response
        return [200, true];
      });
  }
}
