import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import {
  BehaviorSubject,
  filter,
  map,
  Observable,
  of,
  switchMap,
  take,
  tap,
  throwError,
} from 'rxjs';

import { Recommendation } from '../models/recommendation';
import { RecommendationPagination } from '../models/recommendation-pagination';

@Injectable({
  providedIn: 'root',
})
export class RecommendationService {
  // Private
  private __pagination = new BehaviorSubject<
    RecommendationPagination | undefined
  >(undefined);
  private __recommendation = new BehaviorSubject<Recommendation | undefined>(
    undefined
  );
  private __recommendations = new BehaviorSubject<Recommendation[] | undefined>(
    undefined
  );

  /**
   * Constructor
   */
  constructor(private _httpClient: HttpClient) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Accessors
  // -----------------------------------------------------------------------------------------------------

  /**
   * Getter for pagination
   */
  get pagination$(): Observable<RecommendationPagination | undefined> {
    return this.__pagination.asObservable();
  }

  /**
   * Getter for recommendation
   */
  get recommendation$(): Observable<Recommendation | undefined> {
    return this.__recommendation.asObservable();
  }

  /**
   * Getter for recommendations
   */
  get recommendations$(): Observable<Recommendation[] | undefined> {
    return this.__recommendations.asObservable();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Get recommendations
   *
   *
   * @param page
   * @param size
   * @param sort
   * @param order
   * @param search
   */
  getRecommendations(
    page: number = 0,
    size: number = 10,
    sort: string = 'name',
    order: 'asc' | 'desc' | '' = 'asc',
    search: string = ''
  ): Observable<{
    pagination: RecommendationPagination;
    recommendations: Recommendation[];
  }> {
    return this._httpClient
      .get<{
        pagination: RecommendationPagination;
        recommendations: Recommendation[];
      }>('api/apps/member/recommendation/recommendations', {
        params: {
          page: '' + page,
          size: '' + size,
          sort,
          order,
          search,
        },
      })
      .pipe(
        tap((response) => {
          this.__pagination.next(response.pagination);
          this.__recommendations.next(response.recommendations);
        })
      );
  }

  /**
   * Get product by id
   */
  getRecommendationById(id: string | null): Observable<Recommendation> {
    return this.__recommendations.pipe(
      take(1),
      map((recommendations) => {
        // Find the product
        const recommendation =
          recommendations?.find((item) => item.id === id) || undefined;

        // Update the product
        this.__recommendation.next(recommendation);

        // Return the product
        return recommendation;
      }),
      switchMap((product) => {
        if (!product) {
          return throwError('Could not found product with id of ' + id + '!');
        }

        return of(product);
      })
    );
  }

  /**
   * Create product
   */
  createRecommendation(): Observable<Recommendation> {
    return this.recommendations$.pipe(
      take(1),
      switchMap((recommendations) =>
        this._httpClient
          .post<Recommendation>('api/apps/member/recommendation/product', {})
          .pipe(
            map((newRecommendation) => {
              // Update the recommendations with the new product
              if (!!recommendations) {
                this.__recommendations.next([
                  newRecommendation,
                  ...recommendations,
                ]);
              }

              // Return the new product
              return newRecommendation;
            })
          )
      )
    );
  }
}
